
# Default crawler
# This may change from time to time.
# We do not recommend deriving from DefaultCrawler

from microprediction import SequentialStreamCrawler
DefaultCrawler = SequentialStreamCrawler

